// Example1201A
// Author: Akinori TSuji
#include <Wire.h>
#include "RTClib.h"
#include <SoftwareSerial.h>

// TOCOS TWE-LITE
SoftwareSerial xbee(10, 11); // RX, TX

RTC_DS1307 rtc; // リアルタイムクロックDS1307
DateTime now; // リアルタイムクロックの日付の情報

void setup() {
  // リアルタイムクロックDS1307の初期化
  if (!rtc.begin()) { // RTCの動作開始
    Serial.println("Couldn't find RTC");
  }
  if (!rtc.isrunning()) { // RTCが動作していない場合，ホストPCの時刻をセット
    Serial.println("RTC is NOT running!");
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__))); // 時計にずれがある場合，実行
  }

  // シリアル通信の開始
  Serial.begin(19200);
  xbee.begin(19200);
  xbee.readStringUntil('\n'); // 最初のデータを捨てる
  Serial.readStringUntil('\n'); // 最初のデータを捨てる
}

// addzerot: 0-9に対して，00-09になるよう変換
void addzero(int data) {
  if (data < 10) {
    Serial.print("0");
    Serial.print(data);
  } else {
    Serial.print(data);
  }
}

// rtc_out: 日付(yyyymmdd)，曜日(day)，時刻(hhmmss)の情報を表示
void rtc_out() {
  Serial.print(F(":da=")); // date
  Serial.print(now.year());
  addzero(now.month());
  addzero(now.day()); // 曜日 1月 2火 3水 4木 5金 6土 7日
  Serial.print(now.dayOfTheWeek());
  Serial.print(F(":ti=")); // time
  addzero(now.hour());   // hh時間
  addzero(now.minute()); // mm分
  addzero(now.second()); // ss秒
}

String str; // センサからの情報を文字列として格納
int received = 0; // データ受信時のみ更新

void loop() {
  if (xbee.available()) { // 無線よりデータを受信したら
    str = xbee.readStringUntil('\n'); // 改行までデータを読み込む
    xbee.flush(); // バッファをフラッシュ
    received = 1; // 受信フラグをON
  }

  now = rtc.now(); // 日付・時刻の取得
  if ((now.second() % 5) == 0 && received == 1) { // 5秒毎に処理
    rtc_out(); // 時刻情報の表示
    Serial.println(str); // センサからの情報の表示
    Serial.flush(); // バッファをフラッシュ
    received = 0; // 受信フラグをOFF
  }
}

