/*
 * Example1201P
 * Author: Akinori Tsuji
 */
import se.goransson.mqtt.*;
import processing.serial.*;

MQTT mqtt;
Serial myPort;

void setup() {
  myPort = new Serial(this, "COM5", 19200); // COMn を環境に合わせて書き換える
  myPort.bufferUntil('\n');

  mqtt = new MQTT( this );
  mqtt.connect( "web.is.tokushima-u.ac.jp", 1883, "mosquitto-unique-id" );
}


volatile int received = 0; // データの受信が完了した

int N = 12;
float x[] = new float[N];
int n = 0;
float sum = 0;
float TM_AVE_1M; // 1分間の平均気温

void draw() {
  if (received == 1) { // 5秒毎にデータ受信
    //print("sec=");
    //print(second); print(",");
    //date_out();
    if (second == 0) { // 00分からデータをサンプル
      n=0;
      sum=0;
      //println("start sampling");
    }
    x[n] = TM;
    sum += x[n];
    //print("n=");
    //print(n); print(",");
    //print("x(n)=");
    //println(x[n]);
    //print("sum=");
    //println(sum);

    n++;
    if (n >= N) {
      for (int i=0; i<N; i++) println(x[i]);
      TM_AVE_1M = sum / N;
      date_out();
      println(floatFormat(TM_AVE_1M, 1)); // 1分間の平均気温
      n=0; 
      sum=0;
    }
    received = 0;
  }
}

String[] days_week = new String[7];

void date_out() {
  String[] days_week = {"Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"};
  print(year); 
  print("/"); 
  print(month); 
  print("/");
  print(day);
  print("(");
  print(days_week[days-1]);
  print(")");
  print(hour);
  print(":");
  print(minute);
  print(":");
  print(second);
  print(",");
}

//--------------------------------------------------------------------------------------------
// 浮動小数点の数値numを，小数点以下，下n桁に変換
float floatFormat(float num, int n ) {
  float _pow = pow( 10, n );
  return round( num * _pow ) / _pow ; // 小数点以下四捨五入
}

//--------------------------------------------------------------------------------------------
// シリアル通信によって得られたデータの処理（シリアル受信割り込み）
volatile String ED;
volatile float ID, BA, A1, A2, TM, HM, AT;
volatile int SW;
volatile int LX;
volatile long DA, TI;
volatile int year, month, day, days, hour, minute, second;

void serialEvent(Serial myPort) {
  String myString = myPort.readStringUntil('\n'); // 改行コードまでデータ（文字列）を読み込む
  // 文字列が存在していたら
  if (myString != null) {
    myString = trim(myString); // 空白が含まれていたら削除
    //    println(myString);
    String Data1[] = split(myString, ':'); // ':'で文字列を分離して，Data1に格納する
    // データ長（要素）が９以上であれば
    if (Data1.length >= 11) { // Data1[0]は空
      // 文字列Data1の各要素を'='で分離して，各要素を確認する
      String da[] = split(Data1[1], '='); // 日付(yyyymmdd(d)
      if (da[0].equals("da")) {
        DA = Long.valueOf(da[1]);
        year  = (int)(DA / 100000);
        month = (int)(DA / 1000) - year * 100;
        day   = (int)(DA / 10) - year * 10000 - month * 100;
        days  = (int)(DA - year * 100000 - month * 1000 - day*10);
      }
      String ti[] = split(Data1[2], '='); // 時刻(hhmmss)
      if (ti[0].equals("ti")) {
        TI = Long.valueOf(ti[1]);
        hour   = (int)(TI / 10000);
        minute = (int)(TI / 100) - hour*100;
        second = (int)(TI - minute*100 - hour*10000);
      }
      String ed[] = split(Data1[3], '='); // 子機の識別番号（MACアドレス下8桁）
      if (ed[0].equals("ed")) {
        ED = ed[1];
      }      
      String id[] = split(Data1[4], '='); // 子機のID番号
      if (id[0].equals("id")) {
        ID = float(id[1]);
      }    
      String ba[] = split(Data1[5], '='); // 電源電圧（電池の電圧）(mV)
      if (ba[0].equals("ba")) {
        BA = float(ba[1]);
      }
      String a1[] = split(Data1[6], '='); // スイッチの電圧(mV)
      if (a1[0].equals("a1")) {
        A1 = float(a1[1]); // 0-2400 mV
        if (A1 > 1000) { // スイッチの電圧が1000mV以下でスイッチON
          SW = 0; // OFF
        } else {
          SW = 1; // ON
        }
      }
      String a2[] = split(Data1[7], '='); // 照度センサの電圧(mV)
      if (a2[0].equals("a2")) {
        A2 = float(a2[1]); // 0-2400 mV
        //          LX = 2,22 * (A2 * 1000 / R); // A2 * 1000 [uV], R = 10,000 [Ohm]
        //        LX = 2.22 * (A2 * 1000.0/ 10000.0);
        LX = (int)(0.222 * A2); // lux (0[lux] - 532.8[lux])
      }      
      String tm[] = split(Data1[8], '='); // 温度 * 100 (度)
      if (tm[0].equals("tm")) {
        TM = float(tm[1]) / 100;
      }
      String hm[] = split(Data1[9], '='); // 湿度 * 100 (%RH)
      if (hm[0].equals("hu")) {
        HM = float(hm[1]) / 100;
      }
      String at[] = split(Data1[10], '='); // 大気圧 (hPa)
      if (at[0].equals("at")) {
        AT = float(at[1]);
      }


      //println(DA);
      //println(year);
      //println(month);
      //println(day);
      //println(days);
      //println(TI);
      //println(hour);
      //println(minute);
      //println(second);
      //println(ED);
      //println(ID);
      //println(BA);
      //println(SW);
      //println(LX);
      //println(TM);
      //println(HM);
      //println(AT);
      //println();

      // MQTTで送信するメッセージをJSON形式で作成する
      String message = "{";
      message += "\"temp\": "  + str(TM) + ", ";
      message += "\"humid\": " + str(HM) + ", ";
      message += "\"alt\": "   + str(AT) + ", ";
      message += "\"sw\": "    + str(SW) + ", ";
      message += "\"lux\": "   + str(LX);
      message += "}";

      // MATTサーバにトピック(/koukai)をつけてメッセージを発行する
      mqtt.publish("/koukai", message);

      received = 1; // データ受信完了，MQTT送信完了

      myPort.clear();
    }
  }
}